@@ -154,31 +154,61 @@ gtk_source_completion_words_buffer_init (GtkSourceCompletionWordsBuffer *self)
 	                                           (GDestroyNotify)proposal_cache_free);
 }
 
+/* Starts the scanning at @start, and ends at the line end or @end. So at most
+ * one line is scanned, and the text after @end will never be scanned.
+ */
 static GSList *
 scan_line (GtkSourceCompletionWordsBuffer *buffer,
-           GtkTextIter                    *start)
+	   const GtkTextIter              *start,
+	   const GtkTextIter              *end)
 {
-	GtkTextIter end = *start;
-	gchar *text_line;
+	GtkTextIter line_end;
+	GtkTextIter text_end;
+	GtkTextIter check_start;
+	GtkTextIter check_end;
+	gchar *text;
 	GSList *words;
 
-	if (!gtk_text_iter_starts_line (start))
+	if (gtk_text_iter_compare (end, start) <= 0 ||
+	    gtk_text_iter_ends_line (start))
 	{
-		g_warning ("scan line: 'start' doesn't start a line.");
-		gtk_text_iter_set_line_offset (start, 0);
+		return NULL;
 	}
 
-	gtk_text_iter_forward_to_line_end (&end);
+	line_end = *start;
+	gtk_text_iter_forward_to_line_end (&line_end);
 
-	text_line = gtk_text_buffer_get_text (buffer->priv->buffer,
-					      start,
-					      &end,
-					      FALSE);
+	if (gtk_text_iter_compare (end, &line_end) < 0)
+	{
+		text_end = *end;
+	}
+	else
+	{
+		text_end = line_end;
+	}
 
-	words = _gtk_source_completion_words_utils_scan_words (text_line,
-							       buffer->priv->minimum_word_size);
+	check_start = *start;
+	check_end = text_end;
+	_gtk_source_completion_words_utils_adjust_region (&check_start, &check_end);
 
-	g_free (text_line);
+	if (!gtk_text_iter_equal (start, &check_start))
+	{
+		g_warning ("words completion scan_line(): 'start' iter not adjusted.");
+	}
+
+	if (!gtk_text_iter_equal (&text_end, &check_end))
+	{
+		g_warning ("words completion scan_line(): 'text_end' iter not adjusted.");
+	}
+
+	text = gtk_text_buffer_get_text (buffer->priv->buffer,
+					 start,
+					 &text_end,
+					 FALSE);
+
+	words = _gtk_source_completion_words_utils_scan_words (text, buffer->priv->minimum_word_size);
+
+	g_free (text);
 	return words;
 }
 
@@ -241,49 +271,48 @@ add_words (GtkSourceCompletionWordsBuffer *buffer,
 	g_slist_free (words);
 }
 
-/* Scan words in the region delimited by @start and @end. @start must be at the
- * beginning of a line, and @end must be at the end of a line. Maximum
- * @max_lines are scanned.
- *
- * Returns the number of lines scanned. @stop is set to the next line of the
- * last scanned line.
+/* Scan words in the region delimited by @start and @end.
+ * Maximum @max_lines are scanned.
+ * Returns the number of lines scanned.
+ * @stop is set to the iter where the scanning has stopped.
  */
 static guint
 scan_region (GtkSourceCompletionWordsBuffer *buffer,
-	     GtkTextIter                    *start,
-	     GtkTextIter                    *end,
+	     const GtkTextIter              *start,
+	     const GtkTextIter              *end,
 	     guint                           max_lines,
 	     GtkTextIter                    *stop)
 {
+	GtkTextIter iter = *start;
 	guint nb_lines_scanned = 0;
 
 	g_assert (max_lines != 0);
 
-	if (!gtk_text_iter_starts_line (start))
+	while (TRUE)
 	{
-		g_warning ("scan region: 'start' doesn't start a line.");
-		gtk_text_iter_set_line_offset (start, 0);
-	}
+		GSList *words;
 
-	if (!gtk_text_iter_ends_line (end))
-	{
-		g_warning ("scan region: 'end' doesn't end a line.");
-		gtk_text_iter_forward_to_line_end (end);
-	}
+		if (gtk_text_iter_compare (end, &iter) < 0)
+		{
+			*stop = *end;
+			break;
+		}
 
-	while (nb_lines_scanned < max_lines &&
-	       gtk_text_iter_compare (start, end) < 0)
-	{
-		GSList *words = scan_line (buffer, start);
+		if (nb_lines_scanned >= max_lines)
+		{
+			*stop = iter;
+			break;
+		}
+
+		words = scan_line (buffer, &iter, end);
 
 		/* add_words also frees the list */
 		add_words (buffer, words);
 
 		nb_lines_scanned++;
-		gtk_text_iter_forward_line (start);
+		gtk_text_iter_forward_line (&iter);
 	}
 
-	*stop = *start;
 	return nb_lines_scanned;
 }
 
@@ -323,7 +352,6 @@ is_text_region_empty (GtkTextRegion *region)
 static gboolean
 idle_scan_regions (GtkSourceCompletionWordsBuffer *buffer)
 {
-	gboolean finished = FALSE;
 	guint nb_remaining_lines = buffer->priv->scan_batch_size;
 	GtkTextRegionIterator region_iter;
 	GtkTextIter start;
@@ -359,14 +387,13 @@ idle_scan_regions (GtkSourceCompletionWordsBuffer *buffer)
 				  &start,
 				  &stop);
 
-	finished = is_text_region_empty (buffer->priv->scan_region);
-
-	if (finished)
+	if (is_text_region_empty (buffer->priv->scan_region))
 	{
 		buffer->priv->batch_scan_id = 0;
+		return G_SOURCE_REMOVE;
 	}
 
-	return !finished;
+	return G_SOURCE_CONTINUE;
 }
 
 static gboolean
@@ -402,14 +429,14 @@ install_initiate_scan (GtkSourceCompletionWordsBuffer *buffer)
 
 static void
 remove_words_in_subregion (GtkSourceCompletionWordsBuffer *buffer,
-			   GtkTextIter                    *start,
-			   GtkTextIter                    *end)
+			   const GtkTextIter              *start,
+			   const GtkTextIter              *end)
 {
 	GtkTextIter iter = *start;
 
 	while (gtk_text_iter_compare (&iter, end) < 0)
 	{
-		GSList *words = scan_line (buffer, &iter);
+		GSList *words = scan_line (buffer, &iter, end);
 		GSList *item;
 
 		for (item = words; item != NULL; item = g_slist_next (item))
@@ -450,13 +477,13 @@ remove_words_in_region (GtkSourceCompletionWordsBuffer *buffer,
 
 static GtkTextRegion *
 compute_remove_region (GtkSourceCompletionWordsBuffer *buffer,
-		       GtkTextIter                     start,
-		       GtkTextIter                     end)
+		       const GtkTextIter              *start,
+		       const GtkTextIter              *end)
 {
 	GtkTextRegion *remove_region = gtk_text_region_new (buffer->priv->buffer);
 	GtkTextRegionIterator region_iter;
 
-	gtk_text_region_add (remove_region, &start, &end);
+	gtk_text_region_add (remove_region, start, end);
 
 	gtk_text_region_get_iterator (buffer->priv->scan_region,
 				      &region_iter,
@@ -481,29 +508,22 @@ compute_remove_region (GtkSourceCompletionWordsBuffer *buffer,
 	return remove_region;
 }
 
-/* If some lines between @start and @end are not in the scan region, remove the
- * words in those lines.
+/* Remove the words between @start and @end that are not in the scan region.
+ * @start and @end are adjusted to word boundaries, if they touch or are inside
+ * a word.
  */
 static void
 invalidate_region (GtkSourceCompletionWordsBuffer *buffer,
-                   GtkTextIter                    *start,
-                   GtkTextIter                    *end)
+                   const GtkTextIter              *start,
+                   const GtkTextIter              *end)
 {
 	GtkTextIter start_iter = *start;
 	GtkTextIter end_iter = *end;
 	GtkTextRegion *remove_region;
 
-	if (!gtk_text_iter_starts_line (&start_iter))
-	{
-		gtk_text_iter_set_line_offset (&start_iter, 0);
-	}
+	_gtk_source_completion_words_utils_adjust_region (&start_iter, &end_iter);
 
-	if (!gtk_text_iter_ends_line (&end_iter))
-	{
-		gtk_text_iter_forward_to_line_end (&end_iter);
-	}
-
-	remove_region = compute_remove_region (buffer, start_iter, end_iter);
+	remove_region = compute_remove_region (buffer, &start_iter, &end_iter);
 
 	remove_words_in_region (buffer, remove_region);
 	gtk_text_region_destroy (remove_region);
@@ -511,21 +531,13 @@ invalidate_region (GtkSourceCompletionWordsBuffer *buffer,
 
 static void
 add_to_scan_region (GtkSourceCompletionWordsBuffer *buffer,
-                    GtkTextIter                    *start,
-                    GtkTextIter                    *end)
+                    const GtkTextIter              *start,
+                    const GtkTextIter              *end)
 {
 	GtkTextIter start_iter = *start;
 	GtkTextIter end_iter = *end;
 
-	if (!gtk_text_iter_starts_line (&start_iter))
-	{
-		gtk_text_iter_set_line_offset (&start_iter, 0);
-	}
-
-	if (!gtk_text_iter_ends_line (&end_iter))
-	{
-		gtk_text_iter_forward_to_line_end (&end_iter);
-	}
+	_gtk_source_completion_words_utils_adjust_region (&start_iter, &end_iter);
 
 	gtk_text_region_add (buffer->priv->scan_region,
 			     &start_iter,
@@ -556,10 +568,10 @@ on_insert_text_after_cb (GtkTextBuffer                  *textbuffer,
 
 	gtk_text_iter_backward_chars (&start_iter, nb_chars);
 
-	/* If add_to_scan_region() is called before the text insertion, if the
-	 * line is empty, the TextRegion is empty too and it is not added to the
+	/* If add_to_scan_region() is called before the text insertion, the
+	 * created GtkTextRegion can be empty and is thus not added to the
 	 * scan region. After the text insertion, we are sure that the
-	 * TextRegion is not empty and that the words will be scanned.
+	 * GtkTextRegion is not empty and that the words will be scanned.
 	 */
 	add_to_scan_region (buffer, &start_iter, location);
 }
@@ -596,13 +608,14 @@ on_delete_range_after_cb (GtkTextBuffer                  *text_buffer,
 			  GtkTextIter                    *end,
 			  GtkSourceCompletionWordsBuffer *buffer)
 {
-	/* start = end, but add_to_scan_region() moves the iters to the start
-	 * and the end of the line. If the line is empty, the TextRegion won't
-	 * be added to the scan region. If we call add_to_scan_region() when the
-	 * text is not already deleted, the TextRegion is not empty and will be
-	 * added to the scan region, and when the TextRegion becomes empty after
-	 * the text deletion, the TextRegion is not removed from the scan
-	 * region. Hence two callbacks: before and after the text deletion.
+	/* start = end, but add_to_scan_region() adjusts the iters to word
+	 * boundaries if needed. If the adjusted [start,end] region is empty, it
+	 * won't be added to the scan region. If we call add_to_scan_region()
+	 * when the text is not already deleted, the GtkTextRegion is not empty
+	 * and will be added to the scan region, and when the GtkTextRegion
+	 * becomes empty after the text deletion, the GtkTextRegion is not
+	 * removed from the scan region. Hence two callbacks: before and after
+	 * the text deletion.
 	 */
 	add_to_scan_region (buffer, start, end);
 }
@@ -737,8 +750,10 @@ gtk_source_completion_words_buffer_set_minimum_word_size (GtkSourceCompletionWor
 	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_WORDS_BUFFER (buffer));
 	g_return_if_fail (size != 0);
 
-	buffer->priv->minimum_word_size = size;
-
-	remove_all_words (buffer);
-	scan_all_buffer (buffer);
+	if (buffer->priv->minimum_word_size != size)
+	{
+		buffer->priv->minimum_word_size = size;
+		remove_all_words (buffer);
+		scan_all_buffer (buffer);
+	}
 }
