@@ -34,6 +34,10 @@ GSList		*_gtk_source_completion_words_utils_scan_words		(gchar *text,
 G_GNUC_INTERNAL
 gchar		*_gtk_source_completion_words_utils_get_end_word	(gchar *text);
 
+G_GNUC_INTERNAL
+void		 _gtk_source_completion_words_utils_adjust_region	(GtkTextIter *start,
+									 GtkTextIter *end);
+
 G_END_DECLS
 
 #endif /* __GTK_SOURCE_COMPLETION_WORDS_UTILS_H__ */
