@@ -6,6 +6,7 @@
  *                           Chris Phelps <chicane@reninet.com> and
  *                           Jeroen Zwartepoorte <jeroen@xs4all.nl>
  * Copyright (C) 2003 - Paolo Maggi, Gustavo Giráldez
+ * Copyright (C) 2014 - Sébastien Wilmet <swilmet@gnome.org>
  *
  * GtkSourceView is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -56,6 +57,23 @@ typedef enum
 	GTK_SOURCE_BRACKET_MATCH_FOUND
 } GtkSourceBracketMatchType;
 
+/**
+ * GtkSourceChangeCaseType:
+ * @GTK_SOURCE_CHANGE_CASE_LOWER: change case to lowercase.
+ * @GTK_SOURCE_CHANGE_CASE_UPPER: change case to uppercase.
+ * @GTK_SOURCE_CHANGE_CASE_TOGGLE: toggle case of each character.
+ * @GTK_SOURCE_CHANGE_CASE_TITLE: capitalize each word.
+ *
+ * Since: 3.12
+ */
+typedef enum
+{
+	GTK_SOURCE_CHANGE_CASE_LOWER,
+	GTK_SOURCE_CHANGE_CASE_UPPER,
+	GTK_SOURCE_CHANGE_CASE_TOGGLE,
+	GTK_SOURCE_CHANGE_CASE_TITLE
+} GtkSourceChangeCaseType;
+
 struct _GtkSourceBuffer
 {
 	GtkTextBuffer parent_instance;
@@ -175,11 +193,21 @@ gboolean		 gtk_source_buffer_iter_backward_to_context_class_toggle
 										 GtkTextIter		*iter,
 										 const gchar		*context_class);
 
+void			 gtk_source_buffer_change_case				(GtkSourceBuffer        *buffer,
+										 GtkSourceChangeCaseType case_type,
+										 GtkTextIter            *start,
+										 GtkTextIter            *end);
+
 GtkSourceUndoManager	*gtk_source_buffer_get_undo_manager			(GtkSourceBuffer	*buffer);
 
 void			 gtk_source_buffer_set_undo_manager			(GtkSourceBuffer	*buffer,
 										 GtkSourceUndoManager	*manager);
 
+void			 gtk_source_buffer_set_implicit_trailing_newline	(GtkSourceBuffer        *buffer,
+										 gboolean                implicit_trailing_newline);
+
+gboolean		 gtk_source_buffer_get_implicit_trailing_newline	(GtkSourceBuffer        *buffer);
+
 G_END_DECLS
 
 #endif /* __GTK_SOURCE_BUFFER_H__ */
