@@ -19,11 +19,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef GTK_SOURCE_ENCODING_PRIVATE_H
-#define GTK_SOURCE_ENCODING_PRIVATE_H
+#ifndef __GTK_SOURCE_ENCODING_PRIVATE_H__
+#define __GTK_SOURCE_ENCODING_PRIVATE_H__
 
 #include <glib.h>
-#include "gtksourcetypes-private.h"
 
 G_BEGIN_DECLS
 
@@ -43,10 +42,14 @@ typedef enum _GtkSourceEncodingDuplicates
 	GTK_SOURCE_ENCODING_DUPLICATES_KEEP_LAST
 } GtkSourceEncodingDuplicates;
 
-GTK_SOURCE_INTERNAL
+G_GNUC_INTERNAL
 GSList *		_gtk_source_encoding_remove_duplicates		(GSList                      *encodings,
 									 GtkSourceEncodingDuplicates  removal_type);
 
+G_GNUC_INTERNAL
+GSList *		_gtk_source_encoding_get_default_candidates	(void);
+
+
 G_END_DECLS
 
-#endif  /* GTK_SOURCE_ENCODING_PRIVATE_H */
+#endif  /* __GTK_SOURCE_ENCODING_PRIVATE_H__ */
